local HonorTracker = select(2, ...)
local Comms = HonorTracker:RegisterModule("Comms")
local AceSerializer = LibStub("AceSerializer-3.0")
local AceComm = LibStub("AceComm-3.0")

local HONORSPY_COMM_PREFIX = "HonorSpy4"
local HONORTRACKER_COMM_PREFIX = "HonorTracker"
local PLAYER_NAME = UnitName("player")
local PLAYER_SERVER = GetRealmName()
local FULL_PLAYER_NAME = PLAYER_NAME .. PLAYER_SERVER
local CTL = _G.ChatThrottleLib
local SERIALIZE_VERSION_FORMAT = 1

function Comms:OnLoad()
    -- Setup handling HonorSpy data
    local HonorSpy = {}
    HonorSpy.OnMessage = function(self, prefix, msg, distributionType, sender)
        -- Filter out our own messages
        if( FULL_PLAYER_NAME == PLAYER_NAME ) then return end
        -- Filter out cross realm messages
        if( distributionType ~= "GUILD" and UnitRealmRelationship(sender) ~= 1 ) then return end

        local ok, PLAYER_NAME, playerData = AceSerializer:Deserialize(msg);
        if( not ok ) then return end

        HonorTracker:Trigger("OnMessage", sender, "HonorSpy", PLAYER_NAME, playerData)
    end

    AceComm:Embed(HonorSpy)
    HonorSpy:RegisterComm(HONORSPY_COMM_PREFIX, "OnMessage")
end

-- Handle our own data
function Comms:HandleRawData(sender, msg)
    local messageType, unparsedData = string.split(",", msg, 3)
    local data = {AceSerializer:Deserialize(unparsedData)}
    local ok = table.remove(data, 1)
    if( not ok ) then return end

    HonorTracker:Trigger("OnMessage", sender, messageType, unpack(data))
end

function Comms:CHAT_MSG_ADDON(prefix, msg, distributionType, sender)
    -- Filter out other messages not relevant to us
    if( prefix ~= HONORTRACKER_COMM_PREFIX ) then return end
    -- Filter out our own messages
    if( FULL_PLAYER_NAME == PLAYER_NAME ) then return end
    -- Filter out cross realm messages
    if( distributionType ~= "GUILD" and UnitRealmRelationship(sender) ~= 1 ) then return end

    -- Parse HonorTracker messages
    self:HandleRawData(sender, msg)
end

-- Send messages
function Comms:SendHonorSpy(data)
    local msg = AceSerializer:Serialize(data)
    AceComm:SendCommMessage(HONORSPY_COMM_PREFIX, msg, "YELL")
        
    if( GetGuildInfo("player") ) then
        AceComm:SendCommMessage(HONORSPY_COMM_PREFIX, msg, "GUILD")
    end
end

function Comms:SendPrivateMessage(target, type, data)
    local msg = AceSerializer:Serialize(data)
    CTL:SendAddonMessage("BULK", HONORTRACKER_COMM_PREFIX, type .. "," .. msg, "WHISPER", target)
end

function Comms:SendMessage(type, data)
    local msg = AceSerializer:Serialize(data)

    CTL:SendAddonMessage("BULK", HONORTRACKER_COMM_PREFIX, type .. "," .. msg, "YELL")

    if( GetGuildInfo("player") ) then
        CTL:SendAddonMessage("BULK", HONORTRACKER_COMM_PREFIX, type .. "," .. msg, "GUILD")
    end
end

-- Handle compressing and decompressing player data
function Comms:SerializePlayerData(data)
    local serialized = {
        SERIALIZE_VERSION_FORMAT, -- 1
        data.class, -- 2
        data.lastChecked, -- 3
        data.rankPoints, -- 4
        data.rankProgress, -- 5
        data.rank, -- 6
        data.thisWeek.honor, -- 7
        data.thisWeek.kills, -- 8
        data.lastWeek.standing, -- 9
        data.lastWeek.honor -- 10
    }

    return table.concat(serialized, "/")
end

function Comms:DeserializePlayerData(msg)
    local raw = {string.split("/", msg)}
    local data = {
        class = raw[2],
        lastChecked = tonumber(raw[3]),
        rankProgress = tonumber(raw[4]),
        rankPoints = tonumber(raw[5]),
        rank = tonumber(raw[6]),
        thisWeek = {
            honor = tonumber(raw[7]),
            kills = tonumber(raw[8]),
        },
        lastWeek = {
            standing = tonumber(raw[9]),
            honor = tonumber(raw[10]),
        }
    }

    if( type(data) ~= "table" ) then return end
    if( type(data.class) ~= "string" or not RAID_CLASS_COLORS[data.class] ) then return end
    if( type(data.lastChecked) ~= "number" or type(data.rankPoints) ~= "number" or type(data.rankProgress) ~= "number" or type(data.rank) ~= "number") then return end
    if( type(data.thisWeek) ~= "table" or type(data.thisWeek.honor) ~= "number" or type(data.thisWeek.kills) ~= "number" ) then return end
    if( type(data.lastWeek) ~= "table" or type(data.lastWeek.standing) ~= "number" or type(data.lastWeek.honor) ~= "number" ) then return end
    -- Prevent getting a lastChecked too far into the future
    if( data.lastChecked > (GetServerTime() + 600) ) then return end

    return data
end

function Comms:SendPrivatePlayerData(requester, name, data)
    self:SendPrivateMessage(requester, "push", {name, self:SerializePlayerData(data)})
end

function Comms:SendPlayerData(name, withHonorSpy)
    local data = self.realmBracketDB.players[name]
    -- If we've sent the data in the last hour, avoid sending it again to reduce spam
    if( data.lastSent and (GetServerTime() - data.lastSent) <= 3600) then return end
    data.lastSent = GetServerTime()

    if( withHonorSpy and data.thisWeek.kills >= 15 ) then
        local honorSpyFormat = {
            class = data.class,
            last_checked = data.lastChecked,
            RP = data.rankPoints,
            rankProgress = data.rankProgress,
            rank = data.rank,
            thisWeekHonor = data.thisWeek.honor,
            standing = data.lastWeek.standing,
            lastWeekHonor = data.lastWeek.honor
        } 

        self:SendHonorSpy({name, honorSpyFormat})
    end

    self:SendMessage("push", {name, self:SerializePlayerData(data)})
end

local eventFrame = CreateFrame("Frame")
eventFrame:RegisterEvent("CHAT_MSG_ADDON")
eventFrame:SetScript("OnEvent", function(self, event, ...) Comms[event](Comms, ...) end)
